#ifndef MASH_RPI_H
#define MASH_RPI_H

#include "mplug-compat.h"
#include <npapi.h>
#include <stdio.h>

#if defined(XP_UNIX)
#define DEFAULT_SELECT_TIMEOUT 8
#elif defined(XP_WIN)
#define DEFAULT_SELECT_TIMEOUT 15
#endif

class RPI {
public:
	class ArgArray {
	public:
		ArgArray();
		~ArgArray();

		void  Append(const char *list) { Insert(list, argc_); }
		void  Insert(const char *list, int pos);
		void  CreateCmdline(char *&cmdline);
		void  Output(FILE *output);
		int   argc() const { return argc_; }
		const char * const * argv() const { return argv_; }
		
	private:
		void GrowToAtLeast(int newArgc);
		
		int argc_;
		char **argv_;
		int maxArgc_;
	};



	RPI() : sock_(-1) { }
	~RPI();
	
	NPBool NewProcess(ArgArray *args, const char *stdoutFilename,
			  const char *stderrFilename);
	void   DestroyProcess();

	NPBool Notify(const char *string);
	NPBool Notify(int numElems, ...);
	NPBool NotifyInPieces(int numPieces) {
		uint32 tmp=numPieces;
		return Send_(&tmp, sizeof(uint32));
	}
	
	NPBool NotifyNextPiece(int len, const void *buffer);

private:

	NPBool Fork(ArgArray *args, const char *stdoutFilename,
		    const char *stderrFilename);
	NPBool CreateServer();
	int    ServerPort();
	NPBool AcceptConnection();
	NPBool Send_(const void *bytes, uint32 len);
	
	int sock_;
};


#endif /* MASH_RPI_H */
